# -*- coding: utf-8 -*-

import grok

from zope.interface import moduleProvides
from zope.publisher.interfaces import browser

from dolmen.forms.base import IFormLayer
from dolmen.app.layout import interfaces as API
from dolmen.app.layout.master import Resources
from megrok.resourceviewlet import ResourceViewlet


class IBaseLayer(IFormLayer, browser.IDefaultBrowserLayer):
    """Base layer for a dolmen application
    """


class IBaseSkin(IBaseLayer, browser.IBrowserSkinType):
    """A very basic skin, implementing the needed layers to have a fully
    functional display : forms and other components.
    """
    grok.skin("basic_dolmen_skin")


class Resource(ResourceViewlet):
    grok.baseclass()
    grok.viewletmanager(Resources)
    

moduleProvides(API.ISkin)
__all__ = list(API.ISkin)



# -*- coding: utf-8 -*-

from zope.interface import Interface, Attribute


class IDisplayView(Interface):
    """The view of an object. More explicitly, it defines
    a view that displays an object in a non-editable way.
    """


class ISkin(Interface):
    """API - Skin components.
    """
    IBaseLayer = Attribute("Layer used to register all the "
                           "Dolmen centric view components.")
    
    IBaseSkin = Attribute("Skin providing the IBaseLayer. Can"
                          " be applied directly or inherited.")

    Resource = Attribute("Viewlet component used to include resources")


class IContentProviders(Interface):
    """API - Registered content providers.
    """
    Header = Attribute("Viewlet manager involved in rendering the HTML head.")
    Top = Attribute("Viewlet manager for the top part of the body.")
    Footer = Attribute("Viewlet manager for the bottom part of the body.")
    AboveBody = Attribute("Viewlet manager located above the main content.")
    BelowBody = Attribute("Viewlet manager located below the main content.")
    Resources = Attribute("Viewlet manager including resources.")
 

class IGlobalUI(IContentProviders):
    """API - Global user interface components.
    """
    Master = Attribute(
        "Base layout using all the `IContentProviders` components "
        "to build a coherent yet overridable rendering."
        )


class IContextualUI(Interface):
    """API - Pluggable contextual content.
    """
    FlashMessages = Attribute("Viewlet displaying site-wide messages.")
    ContextualActions = Attribute("Viewlet rendering contextual actions.")


class IModels(Interface):
    """API - Available base classes.
    """
    Page = Attribute("Page embedded in a layout.")
    Index = Attribute("Page showing as default view on an object.")
    TablePage = Attribute("Page displaying a table.")
    
    Form = Attribute("Generic page form.")
    SubForm = Attribute("Generic sub-form, used in composed forms.")
    

class IBaseViews(Interface):
    """API - Registered view components.
    """
    Add = Attribute("Default add form.")
    Edit = Attribute("Default edit form.")
    Delete = Attribute("Default delete form.")
    DefaultView = Attribute("Display form used as index.")


class IMenus(Interface):
    """API - Public menu components.
    """
    ContextualMenu = Attribute("Menu defining contextual actions.")
    ContextualMenuEntry = Attribute("Entry of the contextual actions menu.")
    MenuViewlet = Attribute("Generic viewlet rendering a `IBrowserMenu`.")


class IDolmenLayoutAPI(IContentProviders, IGlobalUI, IContextualUI,
                       ISkin, IModels, IBaseViews, IMenus):
    """Dolmen Layout Module API
    """



# -*- coding: utf-8 -*-

import grok
import megrok.menu

from dolmen.app.layout import interfaces as API

from zope.component import getUtility
from zope.interface import moduleProvides
from zope.traversing.browser.absoluteurl import absoluteURL
from zope.browsermenu.interfaces import IBrowserMenu


class ContextualMenu(megrok.menu.Menu):
    grok.name('contextual-actions')
    grok.title('Contextual actions')


class ContextualMenuEntry(object):
    """Defines an contextual actions menu entry.
    """
    grok.baseclass()
    megrok.menu.menuitem(ContextualMenu)


class MenuViewlet(grok.Viewlet):
    grok.baseclass()
    grok.require("dolmen.content.View")

    template = grok.PageTemplateFile("templates/genericmenu.pt")

    menu_class = u"menu"
    entry_class = u"entry"
    actions = []

    @property
    def menu_name(self):
        raise NotImplementedError("You need to specify a menu name.")

    def get_context(self):
        return self.context

    def get_actions(self, context):
        menu = getUtility(IBrowserMenu, self.menu_name)
        actions = menu.getMenuItems(context, self.request)
        return menu.title, actions

    def update(self):
        """Gets the actions and determines which is the selected one.
        """
        context = self.get_context()
        self.title, actions = self.get_actions(context)
        
        if actions:
            url = absoluteURL(context, self.request)
            selected = getattr(self.view, '__name__', None)
            self.actions = [{'url': "%s/%s" % (url, action['action']),
                             'title': action['title'],
                             'selected': action['action'] == selected,
                             'css': (action['action'] == selected
                                     and self.entry_class + ' selected'
                                     or self.entry_class)}
                            for action in actions]


moduleProvides(API.IMenus)
__all__ = list(API.IMenus)



# -*- coding: utf-8 -*-

import grok
from dolmen.app.layout import interfaces as API
from megrok.layout import Layout
from megrok.resourceviewlet import ResourcesManager
from zope.interface import Interface, moduleProvides

grok.templatedir('templates')


class Master(Layout):
    grok.name('master')
    grok.context(Interface)

    def update(self):
        self.base = str(self.request.URL.get(-1))


class Resources(ResourcesManager):
    grok.name('dolmen.resources')
    grok.context(Interface)

        
class Header(grok.ViewletManager):
    grok.name("dolmen.header")
    grok.context(Interface)
    

class Top(grok.ViewletManager):
    grok.name("dolmen.top")
    grok.context(Interface)


class Footer(grok.ViewletManager):
    grok.name("dolmen.footer")
    grok.context(Interface)


class AboveBody(grok.ViewletManager):
    grok.name("dolmen.above.body")
    grok.context(Interface)


class BelowBody(grok.ViewletManager):
    grok.name("dolmen.below.body")
    grok.context(Interface)


moduleProvides(API.IGlobalUI)
__all__ = list(API.IGlobalUI)



# -*- coding: utf-8 -*-

from dolmen.app.layout.interfaces import IDisplayView
from dolmen.app.layout.skin import *
from dolmen.app.layout.menus import *
from dolmen.app.layout.master import *
from dolmen.app.layout.models import *
from dolmen.app.layout.viewlets import *

# APIs
from zope.interface import moduleProvides
from dolmen.app.layout.interfaces import IDolmenLayoutAPI

moduleProvides(IDolmenLayoutAPI)
__all__ = list(IDolmenLayoutAPI) + ['IDisplayView']



# -*- coding: utf-8 -*-

import grok
import megrok.layout
import megrok.z3ctable

from zope.component import getUtility
from zope.interface import moduleProvides

from megrok.z3cform import composed
from z3c.flashmessage.interfaces import IMessageSource

from dolmen.forms import crud
from dolmen.forms.base import PageForm, cancellable

from dolmen.app.site import IDolmen
from dolmen.app.layout import interfaces as API
from dolmen.app.layout import IDisplayView, ContextualMenuEntry


class ApplicationAwareView(object):
    """A mixin allowing to access the application url 
    """
    def application_url(self, name=None):
        """Return the URL of the nearest Dolmen site.
        """
        obj = self.context
        while obj is not None:
            if IDolmen.providedBy(obj):
                return self.url(obj, name)
            obj = obj.__parent__
        raise ValueError("No application found.")


    def flash(self, message, type='message'):
        """Send a short message to the user.
        """
        source = getUtility(IMessageSource, name='session')
        source.send(message, type)


class Page(megrok.layout.Page, ApplicationAwareView):
    """A dolmen site page.
    """
    grok.baseclass()
    grok.require("dolmen.content.View")
    grok.implements(IDisplayView)

    
class TablePage(megrok.z3ctable.TablePage, ApplicationAwareView):
    """A table rendered as a page.
    """
    grok.baseclass()
        

class Index(Page, ContextualMenuEntry):
    """A simple index for dolmen objects.
    """
    grok.baseclass()
    grok.name('index')
    grok.title(crud.i18n(u"View"))
    grok.require("dolmen.content.View")
    grok.implements(IDisplayView)


class DefaultView(crud.Display, ContextualMenuEntry, ApplicationAwareView):
    """The view per default for dolmen contents.
    """
    grok.name('index')
    grok.implements(IDisplayView)
    grok.require("dolmen.content.View")


class Form(PageForm, ApplicationAwareView):
    """A simple dolmen form.
    """
    grok.baseclass()
    cancellable(True)
    grok.require("dolmen.content.View")
    ignoreContext = True


class SubForm(composed.SubForm, ApplicationAwareView):
    """A SubForm base class with a nice template.
    """
    grok.baseclass()

    
class Add(crud.Add, ApplicationAwareView):
    """A generic form to add contents.
    """
    cancellable(True)
    

class Edit(crud.Edit, ContextualMenuEntry, ApplicationAwareView):
    """A generic form to edit contents.
    """
    grok.order(20)
    cancellable(True)
    grok.require("dolmen.content.Edit")


class Delete(crud.Delete, ContextualMenuEntry, ApplicationAwareView):
    """A delete form.
    """
    grok.order(30)
    cancellable(True)
    grok.require("dolmen.content.Delete")

    @property
    def successMessage(self):
        self.flash(crud.Delete.successMessage)
        return crud.Delete.successMessage.fget(self)

    @property
    def failureMessage(self):
        self.flash(crud.Delete.failureMessage)
        return crud.Delete.failureMessage


moduleProvides(API.IModels, API.IBaseViews)
__all__ = list(API.IModels) + list(API.IBaseViews)



# -*- coding: utf-8 -*-

import grok
from dolmen.app.layout import Page
from zope.publisher.interfaces import INotFound

grok.templatedir("templates")


class NotFound(Page):
    grok.name("index.html")
    grok.context(INotFound)
    grok.require("zope.Public")
        
    def update(self):
        self.request.response.setStatus(404)

    def application_url(self, name=None):
        obj = self.context.ob
        while obj is not None:
            if isinstance(obj, grok.Application):
                return self.url(obj, name)
            obj = obj.__parent__
        return self.request.URL.get(0)



from setuptools import setup, find_packages
from os.path import join

name = 'dolmen.app.layout'
version = '0.3.1'
readme = open(join('src', 'dolmen', 'app', 'layout', 'README.txt')).read()
history = open(join('docs', 'HISTORY.txt')).read()

install_requires = [
    'setuptools',
    'grok',
    'grokcore.view',
    'grokcore.viewlet',
    'dolmen.content',
    'dolmen.forms.base',
    'dolmen.forms.crud >= 0.2.1',
    'dolmen.app.site',
    'dolmen.app.security',
    'megrok.menu',
    'megrok.resource',
    'megrok.resourceviewlet',
    'megrok.z3ctable >= 1.3',
    'megrok.z3cform.base',
    'megrok.z3cform.composed',
    'megrok.layout >= 0.9',
    'z3c.flashmessage',
    'zope.component',
    'zope.interface',
    'zope.traversing',
    'zope.publisher',
    'zope.browsermenu'
    ]

tests_require = [
    'zope.security',
    'zope.publisher',
    'zope.securitypolicy',
    'zope.testing',
    'zope.app.testing',
    'zope.app.zcmlfiles',
    ]

setup(name = name,
      version = version,
      description = 'Layout and page models for Dolmen applications',
      long_description = readme + '\n\n' + history,
      keywords = 'Grok Zope3 CMS Dolmen',
      author = 'Souheil Chelfouh',
      author_email = 'trollfot@gmail.com',
      url = 'http://gitweb.dolmen-project.org/',
      download_url = '',
      license = 'GPL',
      packages=find_packages('src', exclude=['ez_setup']),
      package_dir={'': 'src'},
      namespace_packages = ['dolmen', 'dolmen.app'],
      include_package_data = True,
      platforms = 'Any',
      zip_safe = False,
      tests_require = tests_require,
      install_requires = install_requires,
      extras_require = {'test': tests_require},
      test_suite="dolmen.app.layout",
      classifiers = [
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Zope3',
        'Intended Audience :: Other Audience',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
      ],
)



# -*- coding: utf-8 -*-

import grok

from dolmen.content import IBaseContent
from dolmen.app.layout import interfaces as API
from dolmen.app.layout import MenuViewlet, AboveBody, Top

from zope.component import getUtility
from zope.interface import Interface, moduleProvides
from z3c.flashmessage.interfaces import IMessageReceiver


class FlashMessages(grok.Viewlet):
    grok.order(10)
    grok.context(Interface)
    grok.name('dolmen.messages')
    grok.viewletmanager(AboveBody)
    
    def update(self):
        source = getUtility(IMessageReceiver)
        self.messages = list(source.receive())


class ContextualActions(MenuViewlet):
    grok.context(IBaseContent)
    grok.viewletmanager(Top)
    grok.order(50)

    menu_name = u'contextual-actions'

    def get_actions(self, context):
        title, actions = MenuViewlet.get_actions(self, context)
        if len(actions) <= 1:
            return title, []
        return title, actions


moduleProvides(API.IContextualUI)
__all__ = list(API.IContextualUI)
